// ignore_for_file: non_constant_identifier_names, camel_case_types, missing_return, unused_import, unused_local_variable, dead_code, unnecessary_cast
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:foundation_fluttify/foundation_fluttify.dart';
import 'package:tencent_live_fluttify/src/android/android.export.g.dart';
import 'package:tencent_live_fluttify/src/ios/ios.export.g.dart';

import 'enums.dart';

class LivePlayer {
  LivePlayer._();

  /// 创建一个播放器
  ///
  /// 直播和点播
  /// - 直播（LIVE）的视频源是主播实时推送的。因此，主播停止推送后，播放端的画面也会随即停止，而且由于是实时直播，所以播放器在播直播 URL 的时候是没有进度条的。
  /// - 点播（VOD）的视频源是云端的一个视频文件，只要未被从云端移除，视频就可以随时播放， 播放中您可以通过进度条控制播放位置，腾讯视频和优酷、土豆等视频网站上的视频观看就是典型的点播场景。
  ///
  /// 协议的支持
  /// - FLV 优点: 成熟度高、高并发无压力 缺点: 需集成 SDK 才能播放 播放延迟: 2s - 3s
  /// - RTMP 优点: 优质线路下理论延迟最低	缺点: 高并发情况下表现不佳	播放延迟: 1s - 3s
  /// - HLS（m3u8） 优点: 手机浏览器支持度高 缺点: 延迟非常高	播放延迟: 10s - 30s
  static Future<LivePlayer> create() async {
    return platform(
      android: (pool) async {
        final instance = LivePlayer._();

        final context = await android_app_Application.get();
        instance._androidPlayer = await com_tencent_rtmp_TXLivePlayer
            .create__android_content_Context(context);

        return instance;
      },
      ios: (pool) async {
        final instance = LivePlayer._();

        instance._iosPlayer = await TXLivePlayer.create__();

        return instance;
      },
    );
  }

  com_tencent_rtmp_TXLivePlayer _androidPlayer;
  TXLivePlayer _iosPlayer;

  final _androidEventDelegate = _AndroidEventDelegate();
  final _iosEventDelegate = _IOSEventDelegate();

  /// 设置播放界面
  Future<void> setPlayerView(dynamic playerView) async {
    assert(
      playerView is com_tencent_rtmp_ui_TXCloudVideoView ||
          playerView is UIView,
    );
    return platform(
      android: (pool) async {
        await _androidPlayer.setPlayerView(playerView);
      },
      ios: (pool) async {
        // 其首个参数 frame 在 1.5.2 版本后已经被废弃
        final rect = await CGRect.create(0, 0, 0, 0);
        await _iosPlayer.setupVideoWidget_containView_insertIndex(
          rect,
          playerView,
          0,
        );
      },
    );
  }

  /// 开始播放
  Future<void> startPlay(
    String playUrl, {
    PlayType type = PlayType.RTMP,
  }) async {
    return platform(
      android: (pool) async {
        int _type;
        switch (type) {
          case PlayType.RTMP:
            _type = com_tencent_rtmp_TXLivePlayer.PLAY_TYPE_LIVE_RTMP;
            break;
          case PlayType.FLV:
            _type = com_tencent_rtmp_TXLivePlayer.PLAY_TYPE_LIVE_FLV;
            break;
          case PlayType.RTMP_ACC:
            _type = com_tencent_rtmp_TXLivePlayer.PLAY_TYPE_LIVE_RTMP_ACC;
            break;
          case PlayType.HLS:
            _type = com_tencent_rtmp_TXLivePlayer.PLAY_TYPE_VOD_HLS;
            break;
        }
        final result = await _androidPlayer.startPlay(playUrl, _type);
        debugPrint('result: $result');
      },
      ios: (pool) async {
        TX_Enum_PlayType _type;
        switch (type) {
          case PlayType.RTMP:
            _type = TX_Enum_PlayType.PLAY_TYPE_LIVE_RTMP;
            break;
          case PlayType.FLV:
            _type = TX_Enum_PlayType.PLAY_TYPE_LIVE_FLV;
            break;
          case PlayType.RTMP_ACC:
            _type = TX_Enum_PlayType.PLAY_TYPE_LIVE_RTMP_ACC;
            break;
          case PlayType.HLS:
            _type = TX_Enum_PlayType.PLAY_TYPE_VOD_HLS;
            break;
        }
        final result = await _iosPlayer.startPlay_type(playUrl, _type);
        debugPrint('result: $result');
      },
    );
  }

  /// 恢复播放
  Future<void> resumePlay() async {
    return platform(
      android: (pool) => _androidPlayer.resume(),
      ios: (pool) => _iosPlayer.resume(),
    );
  }

  /// 恢复直播
  Future<int> resumeLive() async {
    return platform(
      android: (pool) => _androidPlayer.resumeLive(),
      ios: (pool) => _iosPlayer.resumeLive(),
    );
  }

  /// 是否在播放中
  Future<bool> isPlaying() async {
    return platform(
      android: (pool) => _androidPlayer.isPlaying(),
      ios: (pool) => _iosPlayer.isPlaying(),
    );
  }

  /// 暂停播放
  Future<void> pausePlay() async {
    return platform(
      android: (pool) => _androidPlayer.pause(),
      ios: (pool) => _iosPlayer.pause(),
    );
  }

  /// 停止播放
  Future<void> stopPlay() async {
    return platform(
      android: (pool) => _androidPlayer.stopPlay(true),
      ios: (pool) => _iosPlayer.stopPlay(),
    );
  }

  /// 事件处理
  Future<void> setOnEventListener({
    VoidCallback onWarningVideoDecodeFail,
    VoidCallback onWarningAudioDecodeFail,
    VoidCallback onWarningReconnect,
    VoidCallback onWarningRecvDataLag,
    VoidCallback onWarningVideoPlayLag,
    VoidCallback onWarningHwAccelerationFail,
    VoidCallback onWarningVideoDiscontinuity,
    VoidCallback onWarningDNSFail,
    VoidCallback onWarningServerConnFail,
    VoidCallback onWarningShakeFail,
    VoidCallback onEventRcvFirstIFrame,
    VoidCallback onEventPlayBegin,
  }) async {
    return platform(
      android: (pool) async {
        await _androidPlayer.setPlayListener(
          _androidEventDelegate
            .._onWarningVideoDecodeFail = onWarningVideoDecodeFail
            .._onWarningAudioDecodeFail = onWarningAudioDecodeFail
            .._onWarningReconnect = onWarningReconnect
            .._onWarningRecvDataLag = onWarningRecvDataLag
            .._onWarningVideoPlayLag = onWarningVideoPlayLag
            .._onWarningHwAccelerationFail = onWarningHwAccelerationFail
            .._onWarningVideoDiscontinuity = onWarningVideoDiscontinuity
            .._onWarningDNSFail = onWarningDNSFail
            .._onWarningServerConnFail = onWarningServerConnFail
            .._onWarningShakeFail = onWarningShakeFail
            .._onEventRcvFirstIFrame = onEventRcvFirstIFrame
            .._onEventPlayBegin = onEventPlayBegin,
        );
      },
      ios: (pool) async {
        await _iosPlayer.set_delegate(
          _iosEventDelegate
            .._onWarningVideoDecodeFail = onWarningVideoDecodeFail
            .._onWarningAudioDecodeFail = onWarningAudioDecodeFail
            .._onWarningReconnect = onWarningReconnect
            .._onWarningRecvDataLag = onWarningRecvDataLag
            .._onWarningVideoPlayLag = onWarningVideoPlayLag
            .._onWarningHwAccelerationFail = onWarningHwAccelerationFail
            .._onWarningVideoDiscontinuity = onWarningVideoDiscontinuity
            .._onWarningDNSFail = onWarningDNSFail
            .._onWarningServerConnFail = onWarningServerConnFail
            .._onWarningShakeFail = onWarningShakeFail
            .._onEventRcvFirstIFrame = onEventRcvFirstIFrame
            .._onEventPlayBegin = onEventPlayBegin,
        );
      },
    );
  }
}

class _AndroidEventDelegate extends java_lang_Object
    with com_tencent_rtmp_ITXLivePlayListener {
  VoidCallback _onWarningVideoDecodeFail;
  VoidCallback _onWarningAudioDecodeFail;
  VoidCallback _onWarningReconnect;
  VoidCallback _onWarningRecvDataLag;
  VoidCallback _onWarningVideoPlayLag;
  VoidCallback _onWarningHwAccelerationFail;
  VoidCallback _onWarningVideoDiscontinuity;
  VoidCallback _onWarningDNSFail;
  VoidCallback _onWarningServerConnFail;
  VoidCallback _onWarningShakeFail;

  VoidCallback _onEventRcvFirstIFrame;
  VoidCallback _onEventPlayBegin;

  @override
  Future<void> onPlayEvent(int var1, android_os_Bundle var2) async {
    super.onPlayEvent(var1, var2);
    debugPrint('onPlayEvent: $var1');
    // 当前视频帧解码失败
    if (var1 == 2101 && _onWarningVideoDecodeFail != null) {
      _onWarningVideoDecodeFail();
    }
    // 当前音频帧解码失败
    else if (var1 == 2102 && _onWarningAudioDecodeFail != null) {
      _onWarningAudioDecodeFail();
    }
    // 网络断连，已启动自动重连（重连超过三次就直接抛送 PLAY_ERR_NET_DISCONNECT）
    else if (var1 == 2103 && _onWarningReconnect != null) {
      _onWarningReconnect();
    }
    // 网络来包不稳：可能是下行带宽不足，或由于主播端出流不均匀
    else if (var1 == 2104 && _onWarningRecvDataLag != null) {
      _onWarningRecvDataLag();
    }
    // 当前视频播放出现卡顿
    else if (var1 == 2105 && _onWarningVideoPlayLag != null) {
      _onWarningVideoPlayLag();
    }
    // 硬解启动失败，采用软解
    else if (var1 == 2106 && _onWarningHwAccelerationFail != null) {
      _onWarningHwAccelerationFail();
    }
    // 当前视频帧不连续，可能丢帧
    else if (var1 == 2107 && _onWarningVideoDiscontinuity != null) {
      _onWarningVideoDiscontinuity();
    }
    // RTMP - DNS 解析失败（会触发重试流程） 3001
    else if (var1 == 3001 && _onWarningDNSFail != null) {
      _onWarningDNSFail();
    }
    // RTMP 服务器连接失败（会触发重试流程） 3002
    else if (var1 == 3002 && _onWarningServerConnFail != null) {
      _onWarningServerConnFail();
    }
    // RTMP 服务器握手失败（会触发重试流程） 3003
    else if (var1 == 3003 && _onWarningShakeFail != null) {
      _onWarningShakeFail();
    }
    // 收到首帧数据，越快收到此消息说明链路质量越好
    else if (var1 == 2003 && _onEventRcvFirstIFrame != null) {
      _onEventRcvFirstIFrame();
    }
    // 视频播放开始，如果您自己做 loading，会需要它
    else if (var1 == 2004 && _onEventPlayBegin != null) {
      _onEventPlayBegin();
    }
  }

  @override
  Future<void> onNetStatus(android_os_Bundle var1) async {
    super.onNetStatus(var1);
  }
}

class _IOSEventDelegate extends NSObject with TXLivePlayListener {
  VoidCallback _onWarningVideoDecodeFail;
  VoidCallback _onWarningAudioDecodeFail;
  VoidCallback _onWarningReconnect;
  VoidCallback _onWarningRecvDataLag;
  VoidCallback _onWarningVideoPlayLag;
  VoidCallback _onWarningHwAccelerationFail;
  VoidCallback _onWarningVideoDiscontinuity;
  VoidCallback _onWarningDNSFail;
  VoidCallback _onWarningServerConnFail;
  VoidCallback _onWarningShakeFail;

  VoidCallback _onEventRcvFirstIFrame;
  VoidCallback _onEventPlayBegin;

  @override
  Future<void> onPlayEvent_withParam(int EvtID, Map param) async {
    super.onPlayEvent_withParam(EvtID, param);
    debugPrint('onPlayEvent_withParam: $EvtID, $param');
    // 当前视频帧解码失败
    if (EvtID == 2101 && _onWarningVideoDecodeFail != null) {
      _onWarningVideoDecodeFail();
    }
    // 当前音频帧解码失败
    else if (EvtID == 2102 && _onWarningAudioDecodeFail != null) {
      _onWarningAudioDecodeFail();
    }
    // 网络断连，已启动自动重连（重连超过三次就直接抛送 PLAY_ERR_NET_DISCONNECT）
    else if (EvtID == 2103 && _onWarningReconnect != null) {
      _onWarningReconnect();
    }
    // 网络来包不稳：可能是下行带宽不足，或由于主播端出流不均匀
    else if (EvtID == 2104 && _onWarningRecvDataLag != null) {
      _onWarningRecvDataLag();
    }
    // 当前视频播放出现卡顿
    else if (EvtID == 2105 && _onWarningVideoPlayLag != null) {
      _onWarningVideoPlayLag();
    }
    // 硬解启动失败，采用软解
    else if (EvtID == 2106 && _onWarningHwAccelerationFail != null) {
      _onWarningHwAccelerationFail();
    }
    // 当前视频帧不连续，可能丢帧
    else if (EvtID == 2107 && _onWarningVideoDiscontinuity != null) {
      _onWarningVideoDiscontinuity();
    }
    // RTMP - DNS 解析失败（会触发重试流程） 3001
    else if (EvtID == 3001 && _onWarningDNSFail != null) {
      _onWarningDNSFail();
    }
    // RTMP 服务器连接失败（会触发重试流程） 3002
    else if (EvtID == 3002 && _onWarningServerConnFail != null) {
      _onWarningServerConnFail();
    }
    // RTMP 服务器握手失败（会触发重试流程） 3003
    else if (EvtID == 3003 && _onWarningShakeFail != null) {
      _onWarningShakeFail();
    }
    // 收到首帧数据，越快收到此消息说明链路质量越好
    else if (EvtID == 2003 && _onEventRcvFirstIFrame != null) {
      _onEventRcvFirstIFrame();
    }
    // 视频播放开始，如果您自己做 loading，会需要它
    else if (EvtID == 2004 && _onEventPlayBegin != null) {
      _onEventPlayBegin();
    }
  }

  @override
  Future<void> onNetStatus(Map param) async {
    super.onNetStatus(param);
  }
}
